<?php

namespace Tests\Feature;

use Illuminate\Support\Str;
use Laravel\Sanctum\Sanctum;
use Tests\TestCase;
use App\Models\User;
use App\Models\Product;
use Hash;

class CompareActionTest extends TestCase
{
    public function test_success_action_by_admin()
    {
        $user = User::factory()->create([
            'role' => 'admin',
            'password' => Hash::make('123123123'),
        ]);
        Sanctum::actingAs(
            $user,
            ['*']
        );
        $product = Product::factory()->create();
        $response = $this->post('/nova-api/products/lens/compare-product/action?action=compare-action-change-values&pivotAction=false&filters=W10%3D', [
            'changeModel' => $product->id,
            'changeField' => 'price'
        ]);
        $user->delete();
        $product->delete();
        $response->assertStatus(200);
    }

    public function test_action_without_change_model_by_admin()
    {
        $user = User::factory()->create([
            'role' => 'admin',
            'password' => Hash::make('123123123'),
        ]);
        Sanctum::actingAs(
            $user,
            ['*']
        );
        $response = $this->post('/nova-api/products/lens/compare-product/action?action=compare-action-change-values&pivotAction=false&filters=W10%3D', [
            'changeField' => 'price'
        ]);
        $user->delete();
        $response->assertStatus(302)->assertSessionHasErrors([
            'changeModel' => 'The Model to change field is required.'
        ]);
    }

    public function test_action_without_change_field_by_admin()
    {
        $user = User::factory()->create([
            'role' => 'admin',
            'password' => Hash::make('123123123'),
        ]);
        Sanctum::actingAs(
            $user,
            ['*']
        );
        $product = Product::factory()->create();
        $response = $this->post('/nova-api/products/lens/compare-product/action?action=compare-action-change-values&pivotAction=false&filters=W10%3D', [
            'changeModel' => $product->id,
        ]);
        $user->delete();
        $product->delete();
        $response->assertStatus(302)->assertSessionHasErrors([
            'changeField' => 'The Field to change field is required.'
        ]);
    }

    public function test_success_action_by_analyst()
    {
        $user = User::factory()->create([
            'role' => 'analyst',
            'password' => Hash::make('123123123'),
        ]);
        Sanctum::actingAs(
            $user,
            ['*']
        );
        $product = Product::factory()->create();
        $response = $this->post('/nova-api/products/lens/compare-product/action?action=compare-action-change-values&pivotAction=false&filters=W10%3D', [
            'changeModel' => $product->id,
            'changeField' => 'price'
        ]);
        $user->delete();
        $product->delete();
        $response->assertStatus(200);
    }

    public function test_action_without_change_model_by_analyst()
    {
        $user = User::factory()->create([
            'role' => 'analyst',
            'password' => Hash::make('123123123'),
        ]);
        Sanctum::actingAs(
            $user,
            ['*']
        );
        $response = $this->post('/nova-api/products/lens/compare-product/action?action=compare-action-change-values&pivotAction=false&filters=W10%3D', [
            'changeField' => 'price'
        ]);
        $user->delete();
        $response->assertStatus(302)->assertSessionHasErrors([
            'changeModel' => 'The Model to change field is required.'
        ]);
    }

    public function test_action_without_change_field_by_analyst()
    {
        $user = User::factory()->create([
            'role' => 'analyst',
            'password' => Hash::make('123123123'),
        ]);
        Sanctum::actingAs(
            $user,
            ['*']
        );
        $product = Product::factory()->create();
        $response = $this->post('/nova-api/products/lens/compare-product/action?action=compare-action-change-values&pivotAction=false&filters=W10%3D', [
            'changeModel' => $product->id,
        ]);
        $user->delete();
        $product->delete();
        $response->assertStatus(302)->assertSessionHasErrors([
            'changeField' => 'The Field to change field is required.'
        ]);
    }

    public function test_action_by_common()
    {
        $user = User::factory()->create([
            'role' => 'common',
            'password' => Hash::make('123123123'),
        ]);
        Sanctum::actingAs(
            $user,
            ['*']
        );
        $product = Product::factory()->create();
        $response = $this->post('/nova-api/products/lens/compare-product/action?action=compare-action-change-values&pivotAction=false&filters=W10%3D', [
            'changeModel' => $product->id,
            'changeField' => 'price'
        ]);
        $user->delete();
        $product->delete();
        $response->assertStatus(403);
    }

    public function test_action_by_unauthorized()
    {
        $product = Product::factory()->create();
        $response = $this->post('/nova-api/products/lens/compare-product/action?action=compare-action-change-values&pivotAction=false&filters=W10%3D', [
            'changeModel' => $product->id,
            'changeField' => 'price'
        ]);
        $product->delete();
        $response->assertStatus(401);
    }
}
